@@ -1,9 +1,29 @@
+# switchbot-mqtt - MQTT client controlling SwitchBot button & curtain automators,
+# compatible with home-assistant.io's MQTT Switch & Cover platform
+#
+# Copyright (C) 2020 Fabian Peter Hammerle <fabian@hammerle.me>
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
 import unittest.mock
 
 import pytest
 
 import switchbot_mqtt
 
+# pylint: disable=too-many-arguments; these are tests, no API
+
 
 @pytest.mark.parametrize(
     (
@@ -12,6 +32,7 @@ import switchbot_mqtt
         "expected_mqtt_port",
         "expected_username",
         "expected_password",
+        "expected_retry_count",
     ),
     [
         (
@@ -20,6 +41,7 @@ import switchbot_mqtt
             1883,
             None,
             None,
+            3,
         ),
         (
             ["", "--mqtt-host", "mqtt-broker.local", "--mqtt-port", "8883"],
@@ -27,6 +49,7 @@ import switchbot_mqtt
             8883,
             None,
             None,
+            3,
         ),
         (
             ["", "--mqtt-host", "mqtt-broker.local", "--mqtt-username", "me"],
@@ -34,6 +57,7 @@ import switchbot_mqtt
             1883,
             "me",
             None,
+            3,
         ),
         (
             [
@@ -44,16 +68,24 @@ import switchbot_mqtt
                 "me",
                 "--mqtt-password",
                 "secret",
+                "--retries",
+                "21",
             ],
             "mqtt-broker.local",
             1883,
             "me",
             "secret",
+            21,
         ),
     ],
 )
 def test__main(
-    argv, expected_mqtt_host, expected_mqtt_port, expected_username, expected_password
+    argv,
+    expected_mqtt_host,
+    expected_mqtt_port,
+    expected_username,
+    expected_password,
+    expected_retry_count,
 ):
     with unittest.mock.patch("switchbot_mqtt._run") as run_mock, unittest.mock.patch(
         "sys.argv", argv
@@ -65,11 +97,12 @@ def test__main(
         mqtt_port=expected_mqtt_port,
         mqtt_username=expected_username,
         mqtt_password=expected_password,
+        retry_count=expected_retry_count,
     )
 
 
 @pytest.mark.parametrize(
-    ("password_file_content", "expected_password",),
+    ("password_file_content", "expected_password"),
     [
         ("secret", "secret"),
         ("secret space", "secret space"),
@@ -105,6 +138,7 @@ def test__main_password_file(tmpdir, password_file_content, expected_password):
         mqtt_port=1883,
         mqtt_username="me",
         mqtt_password=expected_password,
+        retry_count=3,
     )
 
 
